function fval = dyn_obj_fun(z,wn,ti,m0,m_ss,well_parameters)
% Objective function based on simulation 

% Input variables:
% z  - decision variables, inputs for considered wells for each time
%       interval
%       z = [z1; z2; z3; ... zn;]
% wn - well numbers in consideration, e.g. [1 2 4 5 7]
% ti - time intervals for inputs in minutes, last element is endtime
%       e.g. [0 60 120 180]
%       minimum start and end time
% m0 - initial states for simulation, [3x7] matrix
% m_ss - Steady-state masses for the system with static optimization
%       optimal inputs
% well_parameters - from init_parameters (to avoid global variable)


par = well_parameters(:,wn);
dt = 30; % Simulation time steps, seconds

% Simulate
fval = 0;
iter = 1;
t = 0;
t_end = 60*ti(end);
in = 1;
next_int = 60*ti(in+1);
u = z(2*(in-1)*length(wn)+1:2*in*length(wn));
m = m0(:,wn);
while t < t_end
    iter = iter+1;
    t = t + dt;
    
    % Simulate all wells one time step
    for w = 1:length(wn);
        well = par(:,w);
        mw = m(:,w);
        uw = u(2*w-1:2*w);
        [mdot, meas] = well_model(mw,uw,well);
        op = (1-par(4,w))*meas(5);
        m(:,w) = max(0,m(:,w) + mdot*dt);
        
        % Update objective function value:
        
%         % Method 1:
%         fval = fval - op;
%         % Method 2:
%         fval = fval + sum(((m(:,w)-m_ss(:,w))./m_ss(:,w)).^2);
        % Method 3:
        fval = fval... 
            + exp(2*t/t_end-2)*sum(((m(:,w)-m_ss(:,w))./m_ss(:,w)).^2)...
            - 0.0003*exp(-10*t/t_end)*op;
    end
    
    % Chek time interval, change inputs
    if t >= next_int
        in = in + 1;
        if in < length(ti)
            next_int = 60*ti(in+1);
            u = z(2*(in-1)*length(wn)+1:2*in*length(wn));
        end
    end
end


    
    
    
    
