% Test script used to calculate well inputs using the different dynamic 
%  optimization methods

clc
tic
global well_parameters general_parameters constraints
init_parameters;

% Load precalculated scenario data
scenario = 'sce2';
sce_ss = [scenario '_ss'];
sce_dyn  = [scenario '_dyn3t'];
s = load(sce_ss);
m0 = s.m0;
zs = s.z;
rs = s.routing;
tis = s.ti;

% Create data for objective and constraint functions
wn = [];
z_end = [];
for i = 1:7
    if zs(2*i-1) > 0
        wn = [wn i];
        z_end = [z_end; zs(2*i-1:2*i)];
    end
end
nw = length(wn);
ti = [0 5 10 15 30 45 60 75 90 105 120];
ni = length(ti)-1;
% Initial value for optimization algorithm
z0 = [];
for i = 1:ni
    z0 = [z0; z_end];
end

constr = constraints;
routing = rs;

% Find steady-state masses
m_ss = [];
for i = 1:nw
    u = z0(2*i-1:2*i);
    well = well_parameters(:,wn(i));
    m_ss = [m_ss steady_state(u,well)];
end

% Set bounds on inputs
available_liftgas = constraints(4)*0.8247/86400; % Converting to kg/s
lb = zeros(2*nw*ni,1);
ub = ones(2*nw*ni,1);
ub(2:2:end) = available_liftgas;

% Set sum gaslift <= available gaslift, Ax<=B
A1 = zeros(ni,2*nw*ni);
for i = 1:ni
    A1(i,2*(i-1)*nw+2:2:2*i*nw) = 1;
end
B1 = ones(ni,1)*available_liftgas;
% Set u_gl <= 10*u_pc (Using a lot of gas-lift )
A2 = zeros(ni*nw,2*nw*ni);
for i = 1:ni*nw
    A2(i,2*i-1:2*i) = [-10 1];
end
B2 = zeros(ni*nw,1);
A = [A1; A2];
B = [B1; B2];
    

% Create function handles for objective function and nonlinear constraints
obj_fun = @(z)dyn_obj_fun(z,wn,ti,m0,m_ss,well_parameters);
% % Method 1:
% nonl_con = @(z)dyn_nonl_con(z,wn,routing,ti,m0,m_ss,...
%     general_parameters,well_parameters,constr);

% Set options
typX = ones(length(z0),1);
typX(1:2:end) = 1;
typX(2:2:end) = 2;
options = optimset(...
    'Display','iter',...
    'Algorithm','sqp',...
    ...'TolFun',1e-3,...
    ...'TolCon',1e-4,...
    'TolX',1e-10,...
    ...'TypicalX',typX,...
    'PlotFcns',{@optimplotfval,@optimplotstepsize},...
    'MaxFunEvals',20000);

% Run fmincon
disp('Running fmincon...');
% Method 1:
% [z_dyn,fval] = fmincon(obj_fun,z0,A,B,[],[],lb,ub,nonl_con,options);
% Method 2 and 3:
[z_dyn,fval] = fmincon(obj_fun,z0,A,B,[],[],lb,ub,[],options);
toc

% Restructure before saving
hrs = ti(end)/60;
z = zeros(14,length(ti)-1);
for i = 1:nw
    z(2*wn(i)-1,:) = z_dyn(2*i-1:2*nw:end)';
    z(2*wn(i),:) = z_dyn(2*i:2*nw:end)';
end
z = [z zs];

r = zeros(7,length(ti));
for i = 1:7
    r(i,:) = routing(i);
end
routing = r;

save(sce_dyn,'m0','z','routing','ti')

























