function [z,fval] = opt_free_solution(well_numbers,routing)
% Function to decide routing and inputs without using optimization

global constraints

n = length(well_numbers);
n1 = sum(routing);
n0 = n - n1;

% Get constraints
G = constraints(1);
L = constraints(2);
W = constraints(3);
GL = constraints(4)*0.8247/86400;
% Divide the liftgas equally between the two flowlines
GL0 = GL/2;
GL1 = GL0;

% Set stability region limits:
aa = [0.19  0.00  0.00  0.00  0.36  0.63  0.20]';
aa = aa(well_numbers);

% Distribute the liftgas equally between the wells in each flowline
z = ones(2*n,1);
for i = 1:n
    if routing(i)
        z(2*i) = GL/2/n1;
    else
        z(2*i) = GL/2/n0;
    end
end

% Get production constraints values
nonlcon = ss_nonl_con(z,well_numbers,routing);
nlcon0 = nonlcon(1:3);
nlcon1 = nonlcon(4:6);

% Stability check (casing heading)
stable = true;
for i = 1:n
    if z(2*i) < aa(i) && z(2*i) > 0
        stable = false;
    end
end
       
nogl = zeros(n,1);
n_nogl0 = 0;
n_nogl1 = 0;
while  ~stable || sum(nlcon0<=0) < 3 || sum(nlcon1<=0) < 3 

    % 1. Find stable inputs

    stable = false;
    while ~stable
        max_ind0 = 0;
        max_aa0 = 0;
        max_ind1 = 0;
        max_aa1 = 0;
        for i = 1:n
            if aa(i) > max_aa0 && nogl(i) < 1 ...
                    && ~routing(i) && z(2*i) < aa(i)
                max_ind0 = i;
                max_aa0 = aa(i);
            elseif aa(i) > max_aa1 && nogl(i) < 1 ...
                    && routing(i) && z(2*i) < aa(i)
                max_ind1 = i;
                max_aa1 = aa(i);
            end
        end
        i0 = max_ind0;
        i1 = max_ind1;
        if i0 > 0
            i = i0;
            nogl(i) = 1;
            z(2*i) = 0;
            aa(i) = 0;
            n_nogl0 = n_nogl0 + 1;
        end
        if i1 > 0
            i = i1;
            nogl(i) = 1;
            z(2*i) = 0;
            aa(i) = 0;
            n_nogl1 = n_nogl1 + 1;
        end
        
        % Redistribute liftgas
        gl0 = GL0/(n0-n_nogl0);
        gl1 = GL1/(n1-n_nogl1);
        stable = true;
        for i = 1:n
            if ~nogl(i)
                if routing(i)
                    z(2*i) = gl1;
                else
                    z(2*i) = gl0;
                end
            end
            % Stability check
            if z(2*i) < aa(i) && z(2*i) > 0
                stable = false;
            end
        end
    end
    
    % 2. Check production limits for each flowline, reduce if necessary
    nonlcon = ss_nonl_con(z,well_numbers,routing);
    nlcon0 = nonlcon(1:3);
    nlcon1 = nonlcon(4:6);
    if sum(nlcon0<=0) < 3
        GL0 = GL0 - 0.01*n0;
        stable = false;
    end
    if sum(nlcon1<=0) < 3
        GL1 = GL1 - 0.01*n1;
        stable = false;
    end
    
end

fval = -ss_obj_fun(z,well_numbers);










