
global well_parameters general_parameters constraints
init_parameters;
clc

fignr = 1;
scenario = 'sce2_';
sce_ss = [scenario 'ss'];
sce_dyn = [scenario 'dyn2t'];


avg_lp = [];
avg_op = [];
avg_wp = [];
avg_gp = [];
op_end = [];
for fignr = 1:2
    % Load scenario
    if fignr == 1
        scenario = sce_ss;
    else
        scenario = sce_dyn;
    end
    sce = load(scenario);
    routing = sce.routing;
    z = sce.z;
    tz = sce.ti;
    m0 = sce.m0;
    % hrs = sce.hrs + 2;
    hrs = 3;

    nt = length(tz);
    dt = 0.5;
    N = hrs*3600/dt;

    m = zeros(3,7,N);
    meas = zeros(11,7,N);
    m(:,:,1) = m0;

    for ti = 1:nt
        if ti == 1
            Nstart = 2;
        else
            Nstart = tz(ti)*60/dt + 1;
        end
        if ti == nt
            Nend = hrs*3600/dt;
        else
            Nend = tz(ti+1)*60/dt;
        end
        for w = 1:7
            u = z(2*w-1:2*w,ti);
            well = well_parameters(:,w);
            for t = Nstart:Nend
                [mdot,y] = well_model(m(:,w,t-1),u,well);
                m(:,w,t) = m(:,w,t-1) + mdot*dt;
                meas(:,w,t) = y;
            end
        end
    end
    meas(:,:,1) = meas(:,:,2);

    lp = [];
    gp = [];
    lp(:,:) = meas(5,:,:)/0.158987295*86400; %Converting to rho*bpd
    gp(:,:) = meas(6,:,:)/0.8247*86400; % Converting to Sm3/day
    r_wc = well_parameters(4,:);
    rho_o = general_parameters(7);
    rho_w = general_parameters(8); 

    gp0 = zeros(1,N);
    gp1 = zeros(1,N);
    lp0 = zeros(1,N);
    lp1 = zeros(1,N);
    wp0 = zeros(1,N);
    wp1 = zeros(1,N);
    op0 = zeros(1,N);
    op1 = zeros(1,N);

    % Inkluder forskjellig routing for forskjellig time interval!
    routing = routing(:,1);

    for i = 1:7
        wp = r_wc(i)*lp(i,:)/rho_w; % Converting to bpd
        op = (1-r_wc(i))*lp(i,:)/rho_o; % Converting to bpd
        lp_bpd = wp + op;
        % for each time interval
        % routing = 
        % samples = 
        % plukk kun ut samples under:
        if routing(i)
            gp1 = gp1 + gp(i,:);
            lp1 = lp1 + lp_bpd;
            wp1 = wp1 + wp;
            op1 = op1 + op;
        else
            gp0 = gp0 + gp(i,:);
            lp0 = lp0 + lp_bpd;
            wp0 = wp0 + wp;
            op0 = op0 + op;
        end
    end
    gp1 = gp1/10;
    gp0 = gp0/10;
    op_end = [op_end op0(end)+op1(end)];

    t = (0:dt:hrs*3600-dt)/3600;
    limits = constraints(1:3);
    limits(1) = limits(1)/10;

    inp = zeros(1,length(tz));
    for i = 1:length(tz)
        inp(i) = sum(z(2:2:end,i));
    end
    inp = [inp inp(end)]/0.8247*8640;

    figure(fignr);

    subplot (5,4,1:3)
    hold off
    plot(t,[gp0+gp1; lp0+lp1; wp0+wp1; op0+op1])
    hold on
    stairs([tz/60 t(end)], inp, 'color', [1 0 1]);
    plot([t(1) t(end)],[5e4 5e4],'--','color', [1 0 1]);
    axis([0 hrs 0 11e4]);
    grid;
    op = mean(op0+op1);
    title('Total production and gas-lift');  %(Oil: ' num2str(op)...
%         ' bpd on avg.)']);
    legend('Gas [10 Sm^3/day]','Liquid [bpd]','Water [bpd]','Oil [bpd]',...
        'Sum gas-lift [10 Sm^3/day]','location',[0.72 0.8 0.27 0.15]);

    subplot(5,4,5:7)
    hold off
    plot(t,[gp0; lp0; wp0; op0]')
    hold on
    plot([t(1) t(end)],[limits limits],'--')
    axis([0 hrs 0 7e4]);
    grid;
    ylabel('Flowline 0');
    legend('Gas [10 Sm^3/day]','Liquid [bpd]','Water [bpd]','Oil [bpd]',...
        'location',[0.72 0.5 0.27 0.2]);

    subplot(5,4,9:11)
    hold off
    plot(t,[gp1; lp1; wp1; op1]')
    hold on
    plot([t(1) t(end)],[limits limits],'--')
    axis([0 hrs 0 7e4]);
    grid;
    ylabel('Flowline 1');
%     legend('Gas [Sm^3/day/10]','Liquid [bpd]','Water [bpd]','Oil [bpd]')

    subplot(5,4,13:15)
    hold off
    stairs([tz/60 hrs],[z(2:2:end,:) z(2:2:end,end)]')
    axis([0 hrs 0 3]);
    grid;
    ylabel('u_g_l [kg/s]');
    legend('Well 1','Well 2','Well 3','Well 4','Well 5','Well 6','Well 7',...
        'location',[0.72 0.15 0.27 0.2]);
    
    subplot(5,4,17:19)
    hold off
    stairs([tz/60 hrs],[z(1:2:end,:) z(1:2:end,end)]')
    axis([0 hrs 0 1.5]);
    grid;
    ylabel('u_p_c [-]');
    xlabel('Time [hrs]');
%     legend('Well 1','Well 2','Well 3','Well 4','Well 5','Well 6','Well 7');
    N2 = 2*3600/dt;
    if fignr == 1
        avg_lp = [avg_lp mean(lp0(1:N2)+lp1(1:N2))];
        avg_op = [avg_op mean(op0(1:N2)+op1(1:N2))];
        avg_wp = [avg_wp mean(wp0(1:N2)+wp1(1:N2))];
        avg_gp = [avg_gp 10*mean(gp0(1:N2)+gp1(1:N2))];
    end
end

avg_op = [avg_op mean(op0(1:N2)+op1(1:N2))]
avg_gp = [avg_gp 10*mean(gp0(1:N2)+gp1(1:N2))]
avg_wp = [avg_wp mean(wp0(1:N2)+wp1(1:N2))]
avg_lp = [avg_lp mean(lp0(1:N2)+lp1(1:N2))]
op_end = op_end
        