function [z,fval] = bbound(obj_fun,z0,A,B,A1,B1,A2,B2,lb,ub,nonlcon,options,UB)

%Recursive function to solve the optimization problem:
% min obj_fun(x)
% s.t.
% 
% lb <= x <= ub
% A*x <= B
% a1*x <= b1 or a2*x <= b2
% (where a1 and a2 are rows of A1 and A2,
%    and b1 and b2 are according elements in B1 and B2.)
% c(x) <= 0
% ceq(x) = 0

% Branch and bound
    % Solve opt problem
    % if not all constraints satisfied
        % Find greatest deviance
        % Branch on this:
            % Branch 1: Solve with A1(i)x <= B1(i)
                % If not all constraints satisfied
                    % RECURSIVE
                % Else
                    % Return solution
            % Branch 2: Solve with A2(i)x <= B2(i)
                % If not all constraints satisfied
                    % RECURSIVE
                % Else
                    % Return solution
            % Compare solutions, return best feasible
    % else
        % Return solution

    % Iteration counter
    global iteration;
    display = false;
    iteration = iteration +1;
    this_iteration = iteration;
    disp(['Node: ' int2str(iteration)]);
        
    % Solve optimization problem
    [z,fval,exitflag] = fmincon(obj_fun,z0,A,B,[],[],lb,ub,nonlcon,options);
    if display
        disp(['fval in this node: ' num2str(fval)]);
    end
    
    if exitflag > 0 % A feasible solution was found
        % Validate solution
        satisfied = true;
        nsat = [];
        n = length(B1);
        for i = 1:n
            if A1(i,:)*z > B1(i) && A2(i,:)*z > B2(i)
                satisfied = false;
                nsat = [nsat; i];
            end
        end

        if ~satisfied
            if display
                disp('Solution at ')
                z
                fval
                disp(['violates constraint(s) ' int2str(nsat')]);
                disp('A1: ');disp(A1);
                disp('B1: ');disp(B1);
            end

            if fval < UB
                % Find greatest deviance
                maxdev = 0;
                for i = 1:length(nsat)
                    j = nsat(i);
                    dev = min(abs(A1(j,:)*z-B1(j)),abs(A2(j,:)*z-B2(j)));
                    if dev > maxdev
                        maxdev = dev;
                        maxind = j;
                    end
                end
                if display
                    disp(['Branching on constraint ' int2str(maxind)]);
                end

                % Branch on constraint with index maxind
                i = maxind;
                a1 = A1(i,:);
                b1 = B1(i);
                a2 = A2(i,:);
                b2 = B2(i);
                all_exc_maxind = [1:(i-1),(i+1):n];
                A1new = A1(all_exc_maxind,:);
                B1new = B1(all_exc_maxind);
                A2new = A2(all_exc_maxind,:);
                B2new = B2(all_exc_maxind);
                Anew1 = [A;a1];
                Bnew1 = [B;b1];
                Anew2 = [A;a2];
                Bnew2 = [B;b2];
                % Branching here (recursive)
                if abs(A1(i,:)*z-B1(i))>abs(A2(i,:)*z-B2(i))
                    [z2 fval2] = bbound(obj_fun,z0,...
                        Anew2,Bnew2,A1new,B1new,A2new,B2new,...
                        lb,ub,nonlcon,options,UB);
                    [z1 fval1] = bbound(obj_fun,z0,...
                        Anew1,Bnew1,A1new,B1new,A2new,B2new,...
                        lb,ub,nonlcon,options,fval2);
                else
                    [z1 fval1] = bbound(obj_fun,z0,...
                        Anew1,Bnew1,A1new,B1new,A2new,B2new,...
                        lb,ub,nonlcon,options,UB);
                    [z2 fval2] = bbound(obj_fun,z0,...
                        Anew2,Bnew2,A1new,B1new,A2new,B2new,...
                        lb,ub,nonlcon,options,fval1);
                end
                % Comparing solutions, returning the best
                
                if display
                    disp(['Branching in node '...
                        int2str(this_iteration) ' gave:'])
                    fval1
                    fval2
                end
                if fval1 < fval2
                    fval = fval1;
                    z = z1;
                else
                    fval = fval2;
                    z = z2;
                end
            else
                if display
                    disp('fval > UB');
                end
                % Not a feasible solution
                fval = 0; 
            end
        else
            if display
                disp('Feasible solution at: ');
                z
                fval
            end
        end
    else
        if display
            disp(['No feasible solution found. Exitflag: '...
                int2str(exitflag)]);
        end
        fval = 0;
    end
    
end
