function [c ceq] = ss_nonl_con(z,well_numbers,routing)
    
    % Production constraints must be given in bpd and Sm3/day
    
    global general_parameters well_parameters ss_data constraints
    rho_o = general_parameters(7);
    rho_w = general_parameters(8);
    wells = well_parameters(:,well_numbers);
    
    if length(routing) ~= length(well_numbers);
        routing = zeros(length(well_numbers),1);
    end
    
    % Initialize return values
    gas0 = 0;
    gas1 = 0;
    liquid0 = 0;
    liquid1 = 0;
    water0 = 0;
    water1 = 0;
    
    % Calculate total production
    for i = 1:length(z)/2
        wn = well_numbers(i);
        u = z(2*i-[1 0]);
        well = wells(:,i);
        ssd.u_pc = ss_data.u_pc;
        ssd.u_gl = ss_data.u_gl;
        ssd.m_ga = ss_data.m_ga(:,:,wn);
        ssd.m_gt = ss_data.m_gt(:,:,wn);
        ssd.m_lt = ss_data.m_lt(:,:,wn);    
        m_ss = ss_fun(u,ssd); % Using precalculated data
%         m_ss = steady_state(u,well); % Using static model
        [~, y] = well_model(m_ss,u,well);
        w_lp = y(5);
        w_gp = y(6);
        r_wc = well(4);
        
        % Liquid in bpd
        liq = (r_wc*w_lp/rho_w + (1-r_wc)*w_lp/rho_o)/0.158987295*86400;
        % Gas in Sm3/day
        gas = w_gp/0.8247*86400;
        % Water in bpd
        wat = r_wc*w_lp/rho_w/0.158987295*86400;
        % Update sums
        if routing(i)
            gas1 = gas1 + gas;
            liquid1 = liquid1 + liq;
            water1 = water1 + wat;
        else
            water0 = water0 + wat;
            liquid0 = liquid0 + liq;
            gas0 = gas0 + gas;
        end 
    end
    
    ceq = [];
    g0 = gas0 - constraints(1);
    l0 = liquid0 - constraints(2);
    w0 = water0 - constraints(3);
    g1 = gas1 - constraints(1);
    l1 = liquid1 - constraints(2);
    w1 = water1 - constraints(3);

    c = [g0 l0 w0 g1 l1 w1]';
    
end