function [z,fval] = ss_optimization_bb(indices,z0,routing,UB)
% Steady-state optimization to find optimal inputs with stability regions

global iteration constraints;

% Gas-lift constraint converted from Sm3/d to kg/s
available_liftgas = constraints(4)*0.8247/86400;

% Get well data
n = length(indices); % Number of wells

% Construct bounds on inputs
lb = zeros(2*n,1);
ub = zeros(2*n,1);
ub(1:2:2*n) = 1;
ub(2:2:2*n) = available_liftgas;

%   Set sum gaslift <= available gaslift, Ax<=B
A = zeros(1,2*n);
A(2:2:2*n) = 1;
B = available_liftgas;
% A = zeros(n+1,2*n);
% A(n+1,2:2:2*n) = 1;
% for i = 1:n
%     A(i,2*i-1) = -30;
%     A(i,2*i) = 1;
% end
% B = [zeros(n,1);available_liftgas];


% Setting up branch conditions:
%   a*u_pc - u_gl <= -b (A1,B1) 
%       OR 
%   u_gl = 0 (A2,B2)
b =  [0.00  0.00  0.00  0.00  0.00  0.33  0.15]';
aa = [0.19  0.00  0.00  0.00  0.36  0.63  0.20]';
a = aa-b;
a = a(indices);
b = b(indices);
A1 = zeros(n,2*n);
A2 = zeros(n,2*n);
for i = 1:n
    A1(i,2*i-1) = a(i);
    A1(i,2*i) = -1;
    A2(i,2*i) = 1;
end
B1 = -b;
B2 = zeros(n,1);

% Optimization

% Create objective function handle
obj_fun = @(z)ss_obj_fun(z,indices);

% Set nonlinear constraints (water/gas production capacities)
nlcon = @(z)ss_nonl_con(z,indices,routing);
%nlcon = [];

% Set options for branch and bound algorithm
opt = optimset('Display','notify','MaxFunEvals',15000,...
   'Algorithm','sqp','TolFun',1e-5,'TolCon',1e-5);
%  (Algorithm: active-set, interior-point, sqp, or trust-region-reflective)
%  (Display: iter, off, final, notify)

% Initialize iteration count
iteration = 0;
% Run branch and bound
[z,fval] = bbound(obj_fun,z0,A,B,A1,B1,A2,B2,lb,ub,nlcon,opt,UB);

fval = -fval;

end






