% Test script to find optimal routing and inputs

clc
%clear all
tic_start = tic;
global constraints
init_parameters;
init_ss_data;


% Set considered wells (index vector, e.g. [1 3 4 6]) 
% well_numbers = [1 5]; % Considered wells in optimization
well_numbers = 1:7;
n = length(well_numbers);

% Set constraints
%constraints = constraints*n/7; %[gas liquid water liftgas]
available_liftgas = constraints(4)*0.8247/86400; % Converting to kg/s

%Routing
bestf = 0;
bestr = [];
% bin = gen_rout_mat(n);
bin = [0 0 1 0 0 0 0];
fvals = zeros(size(bin,1),2);
fvals(:,1) = 1:size(bin,1);
act_con = zeros(size(bin,1),6);

% Starting point for fmincon
z0 = zeros(2*n,1);
z1 = z0;
z2 = z0;
z0(1:2:2*n) = 0.1;
z1(1:2:2*n) = 0.5;
z2(1:2:2*n) = 1;
z2(2:2:2*n) = available_liftgas/n;

fval_bb = 0;
% Try all different routings
for i = 1:size(bin,1)
    tic
    routing = bin(i,:);
    disp(['## Routing ' int2str(i) ' of ' int2str(size(bin,1)) ' #######'])
    
    
    % Solve optimization problem using branch and bound
    disp('Starting point 1 of 3')
    [z_bb0,f0] = ...
        ss_optimization_bb(well_numbers,z0,routing,-fval_bb);
    disp(['fval: ' num2str(f0)])
    if f0 > fval_bb
        fval_bb = f0;
        z_bb = z_bb0;
        bestr = routing;
    end
    disp('Starting point 2 of 3')
    [z_bb1,f1] = ...
        ss_optimization_bb(well_numbers,z1,routing,-fval_bb);
    disp(['fval: ' num2str(f1)])
    if f1 > fval_bb
        fval_bb = f1;
        z_bb = z_bb1;
        bestr = routing;
    end    
    disp('Starting point 3 of 3')
    [z_bb2,f2] = ...
        ss_optimization_bb(well_numbers,z2,routing,-fval_bb);
    disp(['fval: ' num2str(f2)])
    if f2 > fval_bb
        fval_bb = f2;
        z_bb = z_bb2;
        bestr = routing;
    end

    fvals(i,2) = max([f0 f1 f2]);
    toc
    disp(['fval: ' num2str(max([f0 f1 f2]))...
        ' Best fval: ' num2str(fval_bb)]);
end

routing = bestr;
percent_lift_gas_used = sum(z_bb(2:2:end))/available_liftgas*100;

disp('################ Results from optimization ########################')

% Restructure input vector for displaying
% inputs_guess = zeros(2,length(well_numbers));
% inputs_guess(1,:) = z_guess(1:2:end);
% inputs_guess(2,:) = z_guess(2:2:end);


inputs_bb = zeros(2,length(well_numbers));
inputs_bb(1,:) = z_bb(1:2:end);
inputs_bb(2,:) = z_bb(2:2:end);

% fval_guess
% inputs_guess
% nonlcon_guess = ss_nonl_con(z_guess,well_numbers,...
%     constraints(2),constraints(3),[])

fval_bb
inputs_bb
routing
percent_lift_gas_used
nonlcon_bb = ss_nonl_con(z_bb,well_numbers,bestr)
active  = zeros(6,1);
for i = 1:6
    if nonlcon_bb(i) > -0.1
        active(i) = true;
    end
end
active
        
% plot_stability_regions

for i = 1:length(well_numbers)
    well_n = well_numbers(i);
    subplot(2,4,well_n)
%    plot(inputs_guess(1,i),inputs_guess(2,i),'o','Color',[0 0.8 0.4]);
    plot(inputs_bb(1,i),inputs_bb(2,i),'o','Color',[1 0 1]);
end

disp('########################## fvals ##################################')
rout_fvals_act_con =  [fvals act_con]
toc(tic_start)


% Save scenario data
s2 = load('sce2_ss');
s0 = load('sce0_ss');
% m0 = testsce.m0;
z0 = s0.z;
z2 = s2.z;
% z = [z_bb(1:2); 0;0;0;0;0;0; z_bb(3:4); 0;0;0;0;];
% routing = [routing(1); 0; 0; 0; routing(2); 0; 0]; 
m0 = zeros(3,7);
for i = 1:7
    u = z2(2*i-1:2*i);
    well = well_parameters(:,i);
    m0(:,i) = steady_state(u,well);
end

%z = [z0 [z_bb(1:2); 0; 0; z_bb(3:12)]];

z = z0;
routing = s0.routing;
hrs = 2;
ti = 0; 

% save('sce2b_ss','m0','z','routing','hrs','ti');
    
    
