clc

% Function parameters
tol = 0.05;     % Tolerance in estimation of casing heading
pc_step = 0.1;  % Step size for production choke
max_pc = 1;
well_numbers = 1:7; % Considered wells [1..7]
number_of_wells = length(well_numbers);

% Simulation parameters
dt = 0.5;  % Time steps, seconds
hours = 4; % Total simulation time
N = 3600*hours/dt; % Number of time steps

% Initialization
global well_parameters;
init_parameters;
data = [];

for well_number = well_numbers
    
    output = [0 0];
    well = well_parameters(:,well_number);
    %m0 = well(end-2:end);
    
    u_gl = 0.05;
    index = 0;
    for u_pc = pc_step:pc_step:max_pc
        progress = ((well_number-1)*100 + (u_pc-pc_step)*100/max_pc)...
            /number_of_wells;
        disp(['Progress: ' int2str(progress) ' percent']);
 
        h = u_gl + 0.2;
        l = max(0,u_gl-0.05);
        u_gl = (l+h)/2;
        
        while (h-l) > tol
            index = index + 1;
            u = [u_pc u_gl]';
            
            % Find steady state
            m0 = steady_state(u,well);
            % Simulate well while 'measuring' liquid production
            m = m0;
            w_lp = zeros(1,N);            
            for i = 1:N
                [mdot measurements] = well_model(m,u,well);
                w_lp(i) = measurements(5);
                m = m + mdot*dt; %Forward Euler integration
            end
            
            % Cut first part of the simulations
            lp = w_lp(floor(end/100):end);

            % Test: change in variations over time
            lp1 = lp(1:floor(end/2));       % First half
            lp2 = lp(floor(end/2)+1:end);   % Second half
            max1 = max(lp1);
            min1 = min(lp1);
            max2 = max(lp2);
            min2 = min(lp2);
            
            dev_tol = 0.01; % Tolerance in deviation from mean value
            if (max2-min2) > 0.9*(max1-min1)
%                 figure(index);
%                 plot(lp);
%                 title('Not (sufficiently) decreasing variations')
                disp('Not (sufficiently) decreasing variations')
                % Unstable operation detected, increase gas-lift
                l = u_gl;
            else
%                 figure(index);
%                 plot(lp);             
%                 title('Stable')
                disp('Stable')
                % Stable operation detected, reduce gas-lift
                h = u_gl;
            end
            u_gl = (h+l)/2;
        end
        output = [output; u_pc u_gl];
    end
    data = [data output];

end

disp('Analysis complete! Data saved to stability_regions.mat');
save('stability_regions.mat','data');
