function [mdot y] = well_model(m,u,well)

% General parameters

% Commented for increased speed
% global general_parameters
% R     = general_parameters(1);
% g     = general_parameters(2);
% T     = general_parameters(3);
% p_m   = general_parameters(4);
% p_r   = general_parameters(5);
% M_g   = general_parameters(6);
% rho_o = general_parameters(7);
% rho_w = general_parameters(8);
% A_a   = general_parameters(9);
% A_t   = general_parameters(10);
% C     = general_parameters(11);
% C_pc  = general_parameters(12);
% C_iv  = general_parameters(13);
R = 8.3145;
g = 9.81;
T = 350;
p_m = 5e6;
p_r = 25e6;
M_g = 0.0195;
rho_o = 930;
rho_w = 1030;
A_a = 0.02;
A_t = 0.012;
C = 0.8;
C_pc = 0.0016;
C_iv = 0.00016;

% Well specific parameters
V_a   = well(1);
V_t   = well(2);
L_w   = well(3);
r_wc  = well(4);
r_gor = well(5);
Q_max = well(6);

% States and inputs
m_ga = m(1);
m_gt = m(2);
m_lt = m(3);
m_t = m_gt + m_lt;
u_pc = u(1);
u_gl = u(2);

% Extra parameters
T_t   = T;
T_a   = T;
r_glr = (1-r_wc)*r_gor;
rho_l = r_wc*rho_w + (1-r_wc)*rho_o;
nu_l  = 1/rho_l;


% Eikrem
% p_p = R*T_t*m_gt/(M_g*V_t - M_g*nu_l*m_lt);
% p_ti = p_p + g*m_t/A_t;
% p_bh = p_ti + rho_l*g*L_w;
% p_ai = (R*T_a/(M_g*V_a) + g/A_a)*m_ga;
% 
% rho_p = m_t/V_t;
% rho_gi = M_g/R/T_a*p_ai;
% 
% w_p = C_pc*sqrt(rho_p*max(0,p_p-p_m))*u_pc;
% w_lp = m_lt/m_t*w_p;
% w_gp = w_p - w_lp;
% w_lr = rho_l*Q_max*(1 - (1-C)*(p_bh/p_r) - C*(p_bh/p_r)^2);
% w_gr = r_glr*w_lr;
% w_gi = C_iv*sqrt(rho_gi*max(0,p_ai-p_ti));
% w_gl = u_gl;
% 
% mdot_ga = w_gl - w_gi;
% mdot_gt = w_gr + w_gi - w_gp;
% mdot_lt = w_lr - w_lp;


% Binder
p_p = R*T_t*m_gt/(M_g*V_t - M_g*nu_l*m_lt) - g*m_t/2/A_t;
p_ti = p_p + g*m_t/A_t;
a = r_glr*M_g*g*L_w/2/R/T_t;
p_bh = ((1+r_glr+a)*p_ti + rho_l*g*L_w)/(1+r_glr-a);
p_ai = (R*T_a/V_a/M_g + g/2/A_a)*m_ga;

rho_p = rho_l*M_g*p_p*m_t/(rho_l*R*T_t*m_lt + M_g*p_p*m_gt);
rho_gi = M_g/R/T_a*p_ai;

w_p = C_pc*sqrt(rho_p*max(0,p_p-p_m))*u_pc;
w_lp = m_lt/m_t*w_p;
w_gp = w_p - w_lp;
w_lr = rho_l*Q_max*(1 - (1-C)*(p_bh/p_r) - C*(p_bh/p_r)^2);
w_gr = r_glr*w_lr;
w_gi = C_iv*sqrt(rho_gi*max(0,p_ai-p_ti));
w_gl = u_gl;

mdot_ga = w_gl - w_gi;
mdot_gt = w_gr + w_gi - w_gp;
mdot_lt = w_lr - w_lp;

% Return
mdot = [mdot_ga; mdot_gt; mdot_lt];
y = [p_p; p_ti; p_bh; p_ai; w_lp; w_gp; w_lr; w_gr; w_gi; rho_p; rho_gi];
end



